@@ -1,5 +1,6 @@
 from typing import List, Optional, Set, Tuple
 
+from ..failures import ValidationErrorContext
 from ..models import Status
 from ..runner import events
 from ..runner.serialization import SerializedCheck
@@ -21,7 +22,16 @@ def get_unique_failures(checks: List[SerializedCheck]) -> List[SerializedCheck]:
     unique_checks = []
     for check in reversed(checks):
         # There are also could be checks that didn't fail
-        if check.value == Status.failure and (check.name, check.message) not in seen:
-            unique_checks.append(check)
-            seen.add((check.name, check.message))
+        if check.value == Status.failure:
+            key = get_failure_key(check)
+            if (check.name, key) not in seen:
+                unique_checks.append(check)
+                seen.add((check.name, key))
     return unique_checks
+
+
+def get_failure_key(check: SerializedCheck) -> Optional[str]:
+    if isinstance(check.context, ValidationErrorContext):
+        # Deduplicate by JSON Schema path. All errors that happened on this sub-schema will be deduplicated
+        return "/".join(map(str, check.context.schema_path))
+    return check.message
