@@ -1,6 +1,6 @@
 from hashlib import sha1
 from json import JSONDecodeError
-from typing import Any, Callable, Dict, NoReturn, Optional, Type, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, NoReturn, Optional, Type, Union
 
 import attr
 import hypothesis.errors
@@ -8,13 +8,21 @@ import requests
 from jsonschema import ValidationError
 
 from .constants import SERIALIZERS_SUGGESTION_MESSAGE
-from .utils import GenericResponse
+from .failures import FailureContext
+
+if TYPE_CHECKING:
+    from .utils import GenericResponse
 
 
 class CheckFailed(AssertionError):
     """Custom error type to distinguish from arbitrary AssertionError that may happen in the dependent libraries."""
 
     __module__ = "builtins"
+    context: Optional[FailureContext]
+
+    def __init__(self, *args: Any, context: Optional[FailureContext] = None):
+        super().__init__(*args)
+        self.context = context
 
 
 CACHE: Dict[Union[str, int], Type[CheckFailed]] = {}
@@ -47,6 +55,12 @@ def get_grouped_exception(prefix: str, *exceptions: AssertionError) -> Type[Chec
     return _get_hashed_exception("GroupedException", f"{prefix}{message}")
 
 
+def get_server_error(status_code: int) -> Type[CheckFailed]:
+    """Return new exception for the Internal Server Error cases."""
+    name = f"ServerError{status_code}"
+    return get_exception(name)
+
+
 def get_status_code_error(status_code: int) -> Type[CheckFailed]:
     """Return new exception for an unexpected status code."""
     name = f"StatusCodeError{status_code}"
@@ -84,6 +98,11 @@ def get_headers_error(message: str) -> Type[CheckFailed]:
     return _get_hashed_exception("MissingHeadersError", message)
 
 
+def get_timeout_error(deadline: Union[float, int]) -> Type[CheckFailed]:
+    """Request took too long."""
+    return _get_hashed_exception("TimeoutError", str(deadline))
+
+
 @attr.s(slots=True)
 class InvalidSchema(Exception):
     """Schema associated with an API operation contains an error."""
@@ -121,6 +140,10 @@ class DeadlineExceeded(Exception):
         )
 
 
+class UnsupportedSpecification(ValueError):
+    """The API specification is not supported."""
+
+
 class NonCheckError(Exception):
     """An error happened in side the runner, but is not related to failed checks.
 
@@ -131,6 +154,12 @@ class NonCheckError(Exception):
     __module__ = "builtins"
 
 
+class InternalError(Exception):
+    """Internal error in Schemathesis."""
+
+    __module__ = "builtins"
+
+
 SERIALIZATION_NOT_POSSIBLE_MESSAGE = (
     f"Schemathesis can't serialize data to any of the defined media types: {{}} \n{SERIALIZERS_SUGGESTION_MESSAGE}"
 )
@@ -156,7 +185,7 @@ class InvalidRegularExpression(Exception):
 
 @attr.s  # pragma: no mutate
 class HTTPError(Exception):
-    response: GenericResponse = attr.ib()  # pragma: no mutate
+    response: "GenericResponse" = attr.ib()  # pragma: no mutate
     url: str = attr.ib()  # pragma: no mutate
 
     @classmethod
@@ -172,3 +201,7 @@ class HTTPError(Exception):
         # E.g. it will be handled in CLI - a proper error message will be shown
         if 400 <= response.status_code < 600:
             raise cls(response=response, url=schema_path)
+
+
+class UsageError(Exception):
+    """Incorrect usage of Schemathesis functions."""
