@@ -1,5 +1,6 @@
 # pylint: disable=too-many-statements,too-many-branches
 import logging
+import threading
 import time
 import uuid
 from contextlib import contextmanager
@@ -15,6 +16,7 @@ from hypothesis.errors import HypothesisException, InvalidArgument
 from hypothesis_jsonschema._canonicalise import HypothesisRefResolutionError
 from requests.auth import HTTPDigestAuth, _basic_auth_str
 
+from ... import failures
 from ...constants import (
     DEFAULT_STATEFUL_RECURSION_LIMIT,
     RECURSIVE_REFERENCE_ERROR_MESSAGE,
@@ -59,23 +61,39 @@ class BaseRunner:
     stateful_recursion_limit: int = attr.ib(default=DEFAULT_STATEFUL_RECURSION_LIMIT)  # pragma: no mutate
     count_operations: bool = attr.ib(default=True)  # pragma: no mutate
 
-    def execute(self) -> Generator[events.ExecutionEvent, None, None]:
+    def execute(self) -> "EventStream":
         """Common logic for all runners."""
+        event = threading.Event()
+        return EventStream(self._generate_events(event), event)
+
+    def _generate_events(self, stop_event: threading.Event) -> Generator[events.ExecutionEvent, None, None]:
         results = TestResultSet()
 
         initialized = events.Initialized.from_schema(schema=self.schema, count_operations=self.count_operations)
+
+        def _finish() -> events.Finished:
+            return events.Finished.from_results(results=results, running_time=time.monotonic() - initialized.start_time)
+
+        if stop_event.is_set():
+            yield _finish()
+            return
+
         yield initialized
 
+        if stop_event.is_set():
+            yield _finish()
+            return
+
         for event in self._execute(results):
             yield event
-            if (
+            if stop_event.is_set() or (
                 self.exit_first
                 and isinstance(event, events.AfterExecution)
                 and event.status in (Status.error, Status.failure)
             ):
                 break
 
-        yield events.Finished.from_results(results=results, running_time=time.monotonic() - initialized.start_time)
+        yield _finish()
 
     def _execute(self, results: TestResultSet) -> Generator[events.ExecutionEvent, None, None]:
         raise NotImplementedError
@@ -124,6 +142,35 @@ class BaseRunner:
                 yield from handle_schema_error(result.err(), results, data_generation_method, recursion_level)
 
 
+@attr.s(slots=True)  # pragma: no mutate
+class EventStream:
+    """Schemathesis event stream.
+
+    Provides an API to control the execution flow.
+    """
+
+    generator: Generator[events.ExecutionEvent, None, None] = attr.ib()  # pragma: no mutate
+    stop_event: threading.Event = attr.ib()  # pragma: no mutate
+
+    def __next__(self) -> events.ExecutionEvent:
+        return next(self.generator)
+
+    def __iter__(self) -> Generator[events.ExecutionEvent, None, None]:
+        return self.generator
+
+    def stop(self) -> None:
+        """Stop the event stream.
+
+        Its next value will be the last one (Finished).
+        """
+        self.stop_event.set()
+
+    def finish(self) -> events.ExecutionEvent:
+        """Stop the event stream & return the last event."""
+        self.stop()
+        return next(self)
+
+
 def handle_schema_error(
     error: InvalidSchema, results: TestResultSet, data_generation_method: DataGenerationMethod, recursion_level: int
 ) -> Generator[events.ExecutionEvent, None, None]:
@@ -135,6 +182,7 @@ def handle_schema_error(
         result = TestResult(
             method=method,
             path=error.full_path,
+            verbose_name=verbose_name,
             data_generation_method=data_generation_method,
         )
         result.add_error(error)
@@ -179,6 +227,7 @@ def run_test(  # pylint: disable=too-many-locals
     result = TestResult(
         method=operation.method.upper(),
         path=operation.full_path,
+        verbose_name=operation.verbose_name,
         overridden_headers=headers,
         data_generation_method=data_generation_method,
     )
@@ -322,33 +371,33 @@ def run_checks(
         try:
             skip_check = check(response, case)
             if not skip_check:
-                result.add_success(check_name, case, response, elapsed_time)
-                check_results.append(
-                    Check(name=check_name, value=Status.success, response=response, elapsed=elapsed_time, example=case)
-                )
+                check_result = result.add_success(check_name, case, response, elapsed_time)
+                check_results.append(check_result)
         except AssertionError as exc:
             message = str(exc)
             if not message:
                 message = f"Check '{check_name}' failed"
                 exc.args = (message,)
             errors.append(exc)
-            result.add_failure(check_name, case, response, elapsed_time, message)
-            check_results.append(
-                Check(
-                    name=check_name,
-                    value=Status.failure,
-                    response=response,
-                    elapsed=elapsed_time,
-                    example=case,
-                    message=message,
-                )
-            )
+            if isinstance(exc, CheckFailed):
+                context = exc.context
+            else:
+                context = None
+            check_result = result.add_failure(check_name, case, response, elapsed_time, message, context)
+            check_results.append(check_result)
 
     if max_response_time:
         if elapsed_time > max_response_time:
             message = f"Response time exceeded the limit of {max_response_time} ms"
             errors.append(AssertionError(message))
-            result.add_failure("max_response_time", case, response, elapsed_time, message)
+            result.add_failure(
+                "max_response_time",
+                case,
+                response,
+                elapsed_time,
+                message,
+                failures.ResponseTimeExceeded(elapsed=elapsed_time, deadline=max_response_time),
+            )
         else:
             result.add_success("max_response_time", case, response, elapsed_time)
 
@@ -474,11 +523,22 @@ def _network_test(
     request_tls_verify: bool,
     max_response_time: Optional[int],
 ) -> requests.Response:
-    response = case.call(session=session, headers=headers, timeout=timeout, verify=request_tls_verify)
+    check_results: List[Check] = []
+    try:
+        response = case.call(session=session, headers=headers, timeout=timeout, verify=request_tls_verify)
+    except CheckFailed as exc:
+        check_name = "request_timeout"
+        requests_kwargs = case.as_requests_kwargs(base_url=case.get_full_base_url(), headers=headers)
+        request = requests.Request(**requests_kwargs).prepare()
+        elapsed = cast(float, timeout)  # It is defined and not empty, since the exception happened
+        check_result = result.add_failure(
+            check_name, case, None, elapsed, f"Response timed out after {1000 * elapsed:.2f}ms", exc.context, request
+        )
+        check_results.append(check_result)
+        raise exc
     context = TargetContext(case=case, response=response, response_time=response.elapsed.total_seconds())
     run_targets(targets, context)
     status = Status.success
-    check_results: List[Check] = []
     try:
         run_checks(case, checks, check_results, result, response, context.response_time * 1000, max_response_time)
     except CheckFailed:
