@@ -8,8 +8,9 @@ from typing import Any, Dict, List, Optional
 import attr
 import requests
 
+from ..exceptions import FailureContext, InternalError
 from ..models import Case, Check, Interaction, Request, Response, Status, TestResult
-from ..utils import format_exception
+from ..utils import WSGIResponse, format_exception
 
 
 @attr.s(slots=True)  # pragma: no mutate
@@ -20,7 +21,9 @@ class SerializedCase:
     path_template: str = attr.ib()
     path_parameters: Optional[Dict[str, Any]] = attr.ib()
     query: Optional[Dict[str, Any]] = attr.ib()
+    cookies: Optional[Dict[str, Any]] = attr.ib()
     verbose_name: str = attr.ib()
+    media_type: Optional[str] = attr.ib()
 
     @classmethod
     def from_case(cls, case: Case, headers: Optional[Dict[str, Any]]) -> "SerializedCase":
@@ -28,10 +31,12 @@ class SerializedCase:
             text_lines=case.as_text_lines(headers),
             requests_code=case.get_code_to_reproduce(headers),
             curl_code=case.as_curl_command(headers),
-            path_template=case.full_path,
+            path_template=case.path,
             path_parameters=case.path_parameters,
             query=case.query,
+            cookies=case.cookies,
             verbose_name=case.operation.verbose_name,
+            media_type=case.media_type,
         )
 
 
@@ -42,26 +47,39 @@ class SerializedCheck:
     # Check result
     value: Status = attr.ib()  # pragma: no mutate
     request: Request = attr.ib()  # pragma: no mutate
-    response: Response = attr.ib()  # pragma: no mutate
+    response: Optional[Response] = attr.ib()  # pragma: no mutate
     # Generated example
     example: SerializedCase = attr.ib()  # pragma: no mutate
     message: Optional[str] = attr.ib(default=None)  # pragma: no mutate
+    # Failure-specific context
+    context: Optional[FailureContext] = attr.ib(default=None)  # pragma: no mutate
 
     @classmethod
     def from_check(cls, check: Check) -> "SerializedCheck":
-        request = Request.from_prepared_request(check.response.request)
+        if check.response is not None:
+            request = Request.from_prepared_request(check.response.request)
+        elif check.request is not None:
+            # Response is not available, but it is not an error (only time-out behaves this way at the moment)
+            request = Request.from_prepared_request(check.request)
+        else:
+            raise InternalError("Can not find request data")
+
+        response: Optional[Response]
         if isinstance(check.response, requests.Response):
             response = Response.from_requests(check.response)
-        else:
+        elif isinstance(check.response, WSGIResponse):
             response = Response.from_wsgi(check.response, check.elapsed)
+        else:
+            response = None
         headers = {key: value[0] for key, value in request.headers.items()}
-        return SerializedCheck(
+        return cls(
             name=check.name,
             value=check.value,
             example=SerializedCase.from_case(check.example, headers),
             message=check.message,
             request=request,
             response=response,
+            context=check.context,
         )
 
 
@@ -107,6 +125,7 @@ class SerializedInteraction:
 class SerializedTestResult:
     method: str = attr.ib()  # pragma: no mutate
     path: str = attr.ib()  # pragma: no mutate
+    verbose_name: str = attr.ib()  # pragma: no mutate
     has_failures: bool = attr.ib()  # pragma: no mutate
     has_errors: bool = attr.ib()  # pragma: no mutate
     has_logs: bool = attr.ib()  # pragma: no mutate
@@ -121,9 +140,10 @@ class SerializedTestResult:
     @classmethod
     def from_test_result(cls, result: TestResult) -> "SerializedTestResult":
         formatter = logging.Formatter("[%(asctime)s] %(levelname)s in %(module)s: %(message)s")
-        return SerializedTestResult(
+        return cls(
             method=result.method,
             path=result.path,
+            verbose_name=result.verbose_name,
             has_failures=result.has_failures,
             has_errors=result.has_errors,
             has_logs=result.has_logs,
